/**
 *  This file is part of Escudo.
 *
 *  Escudo is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Escudo is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Escudo.  If not, see <http://www.gnu.org/licenses/>.
 */

 
with (EscudoLib) {
  top.Escudo.keyIndicators =  {
    q_account: "SELECT account FROM account ORDER BY account",
    q_payee: "SELECT payee FROM payee ORDER BY payee",
    q_tag: "SELECT tag FROM tag ORDER BY tag",

    indicator_query: null,

    init: function() {
      addObserver("refresh", this.Observer);
      addObserver("invalid", this.Observer);
    },

    Observer : {
      observe: function(subject, topic, data) {
        top.Escudo.keyIndicators.fillTree();
        top.Escudo.keyIndicators.display();

        document.getElementById("indicators-filter").view = top.Escudo.keyIndicators.treeView;
        document.getElementById("indicators-tree-op").view = top.Escudo.keyIndicators.opTreeView;
      }
    },
    
    fillTree: function() {
      // accounts
      var s_account = getStatement(this.q_account);
      var accounts = [];
      while (s_account.step()) { accounts.push([s_account.row.account, true]); }
      this.treeView.refreshAccounts(accounts);
      
      // payees
      var s_payees = getStatement(this.q_payee);
      var payees = [];
      while (s_payees.step()) { payees.push([s_payees.row.payee, true]); }
      this.treeView.refreshPayees(payees);

      // tags
      var s_tags = getStatement(this.q_tag);
      var tags = [];
      while (s_tags.step()) { tags.push([s_tags.row.tag, true]); }
      this.treeView.refreshTags(tags);
    },
    
    createQuery: function() {
      let s = 'SELECT DISTINCT date, strftime("%s",date) * 1000 AS timestamp, ';
      s += " TOTAL(income) AS income, TOTAL(expense) AS expense, total(income - expense) AS balance, ";
      s += " group_concat(DISTINCT P.payee) AS payee, group_concat(DISTINCT T.tag) as tags ";
      
      s += "FROM operation O, account A, payee P, j_opTag J, tag T ";
      s += "WHERE timestamp is not null AND A.id = O.account AND P.id = O.payee AND J.operation = O.id AND J.tag = T.id ";

      var payeesCheck = this.treeView.getPayeeCheck();
      if (payeesCheck.length > 0) {
        s += "AND P.payee IN(" + payeesCheck.join(", ") + ") ";
      } else {
        s += "AND P.payee IS NULL ";
      }

      var accountCheck = this.treeView.getAccountCheck();
      if (accountCheck.length > 0) {
        s += "AND A.account IN(" + accountCheck.join(", ") + ") ";
      } else {
        s += " AND A.account IS NULL ";
      }

      var tagCheck = this.treeView.getTagCheck();
      if (tagCheck.length > 0) {
        s += "AND T.tag IN(" + tagCheck.join(", ") + ") ";
      } else {
        s += " AND T.tag IS NULL ";
      }

      s += "GROUP BY date ORDER BY date";

      this.indicator_query = s;
      return s;
    },
    
    /**
     */
    display: function() {
      this.createQuery();

      var statement = getStatement(this.indicator_query);
 
      var balance = [];
      var expense = [];
      var income = [];
      var ope = [];
      
      var prevAmount = 0;
      while (statement.step()) {
        let timestamp = statement.row.timestamp;
        
        var vIncome = statement.row.income;
        if (vIncome != 0) { income.push([timestamp, vIncome]); }
        
        var vExpense = -statement.row.expense;
        if (vExpense != 0) { expense.push([timestamp, vExpense]); }
        
        var vBalance = statement.row.balance;
        balance.push([timestamp, prevAmount + vBalance]);
        prevAmount += vBalance;

        ope.push([statement.row.date, vIncome, vExpense, statement.row.tags, statement.row.payee]);
      }

      if (balance.length > 0) {
        var last = balance[balance.length - 1];
        balance.push([last[0] + 86400000, last[1]]);
      }

      var values = [balance, expense, income];
      document.getElementById("stat-frame").contentWindow.setData(values);
      this.opTreeView.changeData(ope);
    },

    opTreeView: {
      data: [],
      
      treeBox: null,
      selection: null,
    
      changeData: function(ope) {
        var oldLength = this.data.length;
        var newLength = ope.length;
        this.data = ope;

        var idx = oldLength < newLength ? oldLength : newLength;
        var nb = newLength - oldLength;
        
        if (this.treeBox == null) return;
        this.treeBox.rowCountChanged(idx + 1, nb);
      },
    
      get rowCount()                     { return this.data.length; },
      setTree: function(treeBox)         { this.treeBox = treeBox; },
      getCellText: function(idx, column) { return this.data[idx][column.index]; },
      isContainer: function(idx)         { return false; },
      isContainerOpen: function(idx)     { return false; },
      isContainerEmpty: function(idx)    { return false; },
      isSeparator: function(idx)         { return false; },
      isSorted: function()               { return false; },
      isEditable: function(idx, column)  { return false; },
    
      getParentIndex: function(idx) { return -1; },
      getLevel: function(idx) { return 0; },
      hasNextSibling: function(idx, after) { return idx < this.data.length - 1; },

      setCellValue: function(idx, column, value) { },
      toggleOpenState: function(idx) { },
      getImageSrc: function(idx, column) {},
      getProgressMode : function(idx,column) {},
      getCellValue: function(idx, column) { },
      cycleHeader: function(col) { },
      selectionChanged: function() {},
      cycleCell: function(idx, column) {},
      performAction: function(action) {},
      performActionOnCell: function(action, index, column) {},
      getRowProperties: function(idx, column, prop) {},
      getCellProperties: function(idx, column, prop) {},
      getColumnProperties: function(column, element, prop) {}
    },
    
    treeView : {
      data : {
        Accounts: [],
        Payees: [],
        Tags: []
      },
    
      visibleData : [
        [["Accounts", true], true, false],
        [["Payees", true], true, false],
        [["Tags", true], true, false]
      ],

      treeBox: null,
      selection: null,
    
      get rowCount()                     { return this.visibleData.length; },
      setTree: function(treeBox)         { this.treeBox = treeBox; },
      getCellText: function(idx, column) { return this.visibleData[idx][0][0]; },
      isContainer: function(idx)         { return this.visibleData[idx][1]; },
      isContainerOpen: function(idx)     { return this.visibleData[idx][2]; },
      isContainerEmpty: function(idx)    { return false; },
      isSeparator: function(idx)         { return false; },
      isSorted: function()               { return false; },
      isEditable: function(idx, column)  { return true; },
    
      setCellValue: function(idx, column, value) {
        value = value == "true" ? true : false;
        
        this.visibleData[idx][0][1] = value;
          //if it's a container we change the check state of the visibleData and data
        if (this.visibleData[idx][1] == true) {
          dump(this.visibleData[idx][0][0] + "\n")
          var d = this.data[this.visibleData[idx][0][0]];
          d.forEach(function(element, index, array) { element[1] = value; });

          do {
            this.visibleData[idx][0][1] = value;
            idx += 1;
          } while(idx < this.visibleData.length && this.visibleData[idx][1] == false)
        } else {
          let pIdx = this.getParentIndex(idx);
          let elem = this.data[this.visibleData[pIdx][0][0]][(idx - pIdx - 1)];
          
          elem[1] = value;
        }

        top.Escudo.keyIndicators.display();
      },
    
      getParentIndex: function(idx) {
        if (this.isContainer(idx)) return -1;
        for (var t = idx - 1; t >= 0 ; t--) {
          if (this.isContainer(t)) return t;
        }
        return null;
      },
      getLevel: function(idx) {
        if (this.isContainer(idx)) return 0;
        return 1;
      },
      hasNextSibling: function(idx, after) {
        var thisLevel = this.getLevel(idx);
        for (var t = after + 1; t < this.visibleData.length; t++) {
          var nextLevel = this.getLevel(t);
          if (nextLevel == thisLevel) return true;
          if (nextLevel < thisLevel) break;
        }
        return false;
      },
      toggleOpenState: function(idx) {
        var item = this.visibleData[idx];
        if (!item[1]) return;
    
        if (item[2]) {
          item[2] = false;
    
          var thisLevel = this.getLevel(idx);
          var deletecount = 0;
          for (var t = idx + 1; t < this.visibleData.length; t++) {
            if (this.getLevel(t) > thisLevel) deletecount++;
            else break;
          }
          if (deletecount) {
            this.visibleData.splice(idx + 1, deletecount);
            this.treeBox.rowCountChanged(idx + 1, -deletecount);
          }
        }
        else {
          item[2] = true;
    
          var label = this.visibleData[idx][0];
          var toinsert = this.data[label[0]];
          for (var i = 0; i < toinsert.length; i++) {
            this.visibleData.splice(idx + i + 1, 0, [toinsert[i], false]);
          }
          this.treeBox.rowCountChanged(idx + 1, toinsert.length);
        }
        this.treeBox.invalidateRow(idx);
      },
    
      getImageSrc: function(idx, column) {},
      getProgressMode : function(idx,column) {},
      getCellValue: function(idx, column) { return this.visibleData[idx][0][1]; },
      cycleHeader: function(col, elem) {},
      selectionChanged: function() {},
      cycleCell: function(idx, column) {},
      performAction: function(action) {},
      performActionOnCell: function(action, index, column) {},
      getRowProperties: function(idx, column, prop) {},
      getCellProperties: function(idx, column, prop) {},
      getColumnProperties: function(column, element, prop) {},
      
      //
      refreshAccounts: function(newAccounts) { this.data.Accounts = newAccounts; },
      refreshTags: function(newTags) { this.data.Tags = newTags; },
      refreshPayees: function(newPayees) { this.data.Payees = newPayees; },
      
      getPayeeCheck: function() {
        var ret = [];
        for (var i = 0; i < this.data.Payees.length; i++) {
          if (this.data.Payees[i][1] === true) {
            ret.push("'" + this.data.Payees[i][0].replace("'", "''") + "'");
          }
        }
        return ret;
      },

      getTagCheck: function() {
        var ret = [];
        for (var i = 0; i < this.data.Tags.length; i++) {
          if (this.data.Tags[i][1] === true) {
            ret.push("'" + this.data.Tags[i][0].replace("'", "''") + "'");
          }
        }
        return ret;
      },
      
      getAccountCheck: function() {
        var ret = [];
        for (var i = 0; i < this.data.Accounts.length; i++) {
          if (this.data.Accounts[i][1] === true) {
            ret.push("'" + this.data.Accounts[i][0].replace("'", "''") + "'");
          }
        }
        return ret;
      }
    }
  }
 
  top.Escudo.keyIndicators.init();
}
